<?php

namespace Btravel\Attribute\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Btravel\Attribute\Http\Requests\AttributeCategoryRequest;
use Btravel\Attribute\Interfaces\AttributeCategoryRepositoryInterface;
use Btravel\Attribute\Models\AttributeCategory;

/**
 * Resource controller class for attribute_category.
 */
class AttributeCategoryResourceController extends BaseController
{

    /**
     * Initialize attribute_category resource controller.
     *
     * @param type AttributeCategoryRepositoryInterface $attribute_category
     *
     * @return null
     */
    public function __construct(AttributeCategoryRepositoryInterface $attribute_category)
    {
        parent::__construct();
        $this->repository = $attribute_category;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Btravel\Attribute\Repositories\Criteria\AttributeCategoryResourceCriteria::class);
    }

    /**
     * Display a list of attribute_category.
     *
     * @return Response
     */
    public function index(AttributeCategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Btravel\Attribute\Repositories\Presenter\AttributeCategoryPresenter::class)
                ->$function();
        }

        $attribute_categories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('attribute::attribute_category.names'))
            ->view('attribute::attribute_category.index', true)
            ->data(compact('attribute_categories', 'view'))
            ->output();
    }

    /**
     * Display attribute_category.
     *
     * @param Request $request
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function show(AttributeCategoryRequest $request, AttributeCategory $attribute_category)
    {

        if ($attribute_category->exists) {
            $view = 'attribute::attribute_category.show';
        } else {
            $view = 'attribute::attribute_category.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('attribute::attribute_category.name'))
            ->data(compact('attribute_category'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new attribute_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AttributeCategoryRequest $request)
    {

        $attribute_category = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('attribute::attribute_category.name')) 
            ->view('attribute::attribute_category.create', true) 
            ->data(compact('attribute_category'))
            ->output();
    }

    /**
     * Create new attribute_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AttributeCategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $attribute_category                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('attribute::attribute_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('attribute/attribute_category/' . $attribute_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/attribute/attribute_category'))
                ->redirect();
        }

    }

    /**
     * Show attribute_category for editing.
     *
     * @param Request $request
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function edit(AttributeCategoryRequest $request, AttributeCategory $attribute_category)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('attribute::attribute_category.name'))
            ->view('attribute::attribute_category.edit', true)
            ->data(compact('attribute_category'))
            ->output();
    }

    /**
     * Update the attribute_category.
     *
     * @param Request $request
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function update(AttributeCategoryRequest $request, AttributeCategory $attribute_category)
    {
        try {
            $attributes = $request->all();

            $attribute_category->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('attribute::attribute_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('attribute/attribute_category/' . $attribute_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('attribute/attribute_category/' . $attribute_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the attribute_category.
     *
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function destroy(AttributeCategoryRequest $request, AttributeCategory $attribute_category)
    {
        try {

            $attribute_category->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('attribute::attribute_category.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('attribute/attribute_category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('attribute/attribute_category/' . $attribute_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple attribute_category.
     *
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function delete(AttributeCategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('attribute::attribute_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('attribute/attribute_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/attribute/attribute_category'))
                ->redirect();
        }

    }

    /**
     * Restore deleted attribute_categories.
     *
     * @param Model   $attribute_category
     *
     * @return Response
     */
    public function restore(AttributeCategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('attribute::attribute_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/attribute/attribute_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/attribute/attribute_category/'))
                ->redirect();
        }

    }

}
